@@ -24,6 +24,10 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <config.h>
+
+#include <assert.h>
+
 #define DBG_IS_TOO_VERBOSE
 #include "types-internal.h"
 #include "message-internal.h"
@@ -143,7 +147,7 @@ get_object_path(PyObject *obj)
             return magic_attr;
         }
         else {
-            Py_DECREF(magic_attr);
+            Py_CLEAR(magic_attr);
             PyErr_SetString(PyExc_TypeError, "__dbus_object_path__ must be "
                             "a string");
             return NULL;
@@ -182,10 +186,10 @@ _signature_string_from_pyobject(PyObject *obj, long *variant_level_ptr)
     if (!magic_attr)
         return NULL;
     if (magic_attr != Py_None) {
-        Py_DECREF(magic_attr);
+        Py_CLEAR(magic_attr);
         return PyString_FromString(DBUS_TYPE_OBJECT_PATH_AS_STRING);
     }
-    Py_DECREF(magic_attr);
+    Py_CLEAR(magic_attr);
 
     /* Ordering is important: some of these are subclasses of each other. */
     if (PyInt_Check(obj)) {
@@ -246,22 +250,22 @@ _signature_string_from_pyobject(PyObject *obj, long *variant_level_ptr)
         if (!list) return NULL;
         if (len == 0) {
             PyErr_SetString(PyExc_ValueError, "D-Bus structs cannot be empty");
-            Py_DECREF(list);
+            Py_CLEAR(list);
             return NULL;
         }
         /* Set the first and last elements of list to be the parentheses */
         item = PyString_FromString(DBUS_STRUCT_BEGIN_CHAR_AS_STRING);
         if (PyList_SetItem(list, 0, item) < 0) {
-            Py_DECREF(list);
+            Py_CLEAR(list);
             return NULL;
         }
         item = PyString_FromString(DBUS_STRUCT_END_CHAR_AS_STRING);
         if (PyList_SetItem(list, len + 1, item) < 0) {
-            Py_DECREF(list);
+            Py_CLEAR(list);
             return NULL;
         }
         if (!item || !PyList_GET_ITEM(list, 0)) {
-            Py_DECREF(list);
+            Py_CLEAR(list);
             return NULL;
         }
         item = NULL;
@@ -269,16 +273,16 @@ _signature_string_from_pyobject(PyObject *obj, long *variant_level_ptr)
         for (i = 0; i < len; i++) {
             item = PyTuple_GetItem(obj, i);
             if (!item) {
-                Py_DECREF(list);
+                Py_CLEAR(list);
                 return NULL;
             }
             item = _signature_string_from_pyobject(item, NULL);
             if (!item) {
-                Py_DECREF(list);
+                Py_CLEAR(list);
                 return NULL;
             }
             if (PyList_SetItem(list, i + 1, item) < 0) {
-                Py_DECREF(list);
+                Py_CLEAR(list);
                 return NULL;
             }
             item = NULL;
@@ -286,13 +290,13 @@ _signature_string_from_pyobject(PyObject *obj, long *variant_level_ptr)
         empty_str = PyString_FromString("");
         if (!empty_str) {
             /* really shouldn't happen */
-            Py_DECREF(list);
+            Py_CLEAR(list);
             return NULL;
         }
         ret = PyObject_CallMethod(empty_str, "join", "(O)", list); /* new ref */
         /* whether ret is NULL or not, */
-        Py_DECREF(empty_str);
-        Py_DECREF(list);
+        Py_CLEAR(empty_str);
+        Py_CLEAR(list);
         return ret;
     }
     else if (PyList_Check(obj)) {
@@ -345,8 +349,8 @@ _signature_string_from_pyobject(PyObject *obj, long *variant_level_ptr)
                                       PyString_AS_STRING(keysig),
                                       PyString_AS_STRING(valuesig));
         }
-        Py_XDECREF(keysig);
-        Py_XDECREF(valuesig);
+        Py_CLEAR(keysig);
+        Py_CLEAR(valuesig);
         return ret;
     }
     else {
@@ -399,7 +403,7 @@ dbus_py_Message_guess_signature(PyObject *unused UNUSED, PyObject *args)
         PyErr_SetString(PyExc_RuntimeError, "Internal error: "
                         "_signature_string_from_pyobject returned "
                         "a bad result");
-        Py_DECREF(tmp);
+        Py_CLEAR(tmp);
         return NULL;
     }
     ret = PyObject_CallFunction((PyObject *)&DBusPySignature_Type, "(s#)",
@@ -407,7 +411,7 @@ dbus_py_Message_guess_signature(PyObject *unused UNUSED, PyObject *args)
                                 PyString_GET_SIZE(tmp) - 2);
     DBG("Message_guess_signature: returning Signature at %p \"%s\"", ret,
         ret ? PyString_AS_STRING(ret) : "(NULL)");
-    Py_DECREF(tmp);
+    Py_CLEAR(tmp);
     return ret;
 }
 
@@ -429,7 +433,7 @@ _message_iter_append_string(DBusMessageIter *appender,
         PyObject *object_path = get_object_path (obj);
 
         if (object_path == Py_None) {
-            Py_DECREF(object_path);
+            Py_CLEAR(object_path);
         }
         else if (!object_path) {
             return -1;
@@ -437,7 +441,7 @@ _message_iter_append_string(DBusMessageIter *appender,
         else {
             int ret = _message_iter_append_string(appender, sig_type,
                                                   object_path, FALSE);
-            Py_DECREF(object_path);
+            Py_CLEAR(object_path);
             return ret;
         }
     }
@@ -454,8 +458,7 @@ _message_iter_append_string(DBusMessageIter *appender,
                             "to be sent over D-Bus must be valid UTF-8");
             return -1;
         }
-        Py_DECREF(unicode);
-        unicode = NULL;
+        Py_CLEAR(unicode);
 
         DBG("Performing actual append: string %s", s);
         if (!dbus_message_iter_append_basic(appender, sig_type,
@@ -474,7 +477,7 @@ _message_iter_append_string(DBusMessageIter *appender,
             PyErr_NoMemory();
             return -1;
         }
-        Py_DECREF(utf8);
+        Py_CLEAR(utf8);
     }
     else {
         PyErr_SetString(PyExc_TypeError,
@@ -493,7 +496,7 @@ _message_iter_append_byte(DBusMessageIter *appender, PyObject *obj)
         if (PyString_GET_SIZE(obj) != 1) {
             PyErr_Format(PyExc_ValueError, "Expected a string of "
                          "length 1 byte, but found %d bytes",
-                         PyString_GET_SIZE(obj));
+                         (int) PyString_GET_SIZE(obj));
             return -1;
         }
         y = *(unsigned char *)PyString_AS_STRING(obj);
@@ -517,6 +520,22 @@ _message_iter_append_byte(DBusMessageIter *appender, PyObject *obj)
     return 0;
 }
 
+static dbus_bool_t
+dbuspy_message_iter_close_container(DBusMessageIter *iter,
+                                    DBusMessageIter *sub,
+                                    dbus_bool_t is_ok)
+{
+#ifdef HAVE_DBUS_MESSAGE_ITER_ABANDON_CONTAINER
+    if (!is_ok) {
+        dbus_message_iter_abandon_container(iter, sub);
+        return TRUE;
+    }
+#else
+    (void) is_ok;
+#endif
+    return dbus_message_iter_close_container(iter, sub);
+}
+
 static int
 _message_iter_append_dictentry(DBusMessageIter *appender,
                                DBusSignatureIter *sig_iter,
@@ -563,12 +582,12 @@ _message_iter_append_dictentry(DBusMessageIter *appender,
         ret = _message_iter_append_pyobject(&sub, &sub_sig_iter, value, &more);
     }
     DBG("%s", "Closing DICT_ENTRY container");
-    if (!dbus_message_iter_close_container(appender, &sub)) {
+    if (!dbuspy_message_iter_close_container(appender, &sub, (ret == 0))) {
         PyErr_NoMemory();
         ret = -1;
     }
 out:
-    Py_DECREF(value);
+    Py_CLEAR(value);
     return ret;
 }
 
@@ -588,6 +607,9 @@ _message_iter_append_multi(DBusMessageIter *appender,
     int inner_type;
     dbus_bool_t more;
 
+    assert(mode == DBUS_TYPE_DICT_ENTRY || mode == DBUS_TYPE_ARRAY ||
+            mode == DBUS_TYPE_STRUCT);
+
 #ifdef USING_DBG
     fprintf(stderr, "Appending multiple: ");
     PyObject_Print(obj, stderr, 0);
@@ -630,6 +652,7 @@ _message_iter_append_multi(DBusMessageIter *appender,
         goto out;
     }
     ret = 0;
+    more = TRUE;
     while ((contents = PyIter_Next(iterator))) {
 
         if (mode == DBUS_TYPE_ARRAY || mode == DBUS_TYPE_DICT_ENTRY) {
@@ -647,6 +670,14 @@ _message_iter_append_multi(DBusMessageIter *appender,
             }
 #endif
         }
+        else /* struct */ {
+            if (!more) {
+                PyErr_Format(PyExc_TypeError, "Fewer items found in struct's "
+                             "D-Bus signature than in Python arguments ");
+                ret = -1;
+                break;
+            }
+        }
 
         if (mode == DBUS_TYPE_DICT_ENTRY) {
             ret = _message_iter_append_dictentry(&sub_appender, &sub_sig_iter,
@@ -665,11 +696,11 @@ _message_iter_append_multi(DBusMessageIter *appender,
             if (!args)
                 break;
             byte = PyObject_Call((PyObject *)&DBusPyByte_Type, args, NULL);
-            Py_DECREF(args);
+            Py_CLEAR(args);
             if (!byte)
                 break;
             ret = _message_iter_append_variant(&sub_appender, byte);
-            Py_DECREF(byte);
+            Py_CLEAR(byte);
         }
         else {
             /* advances sub_sig_iter and sets more on success - for array
@@ -678,7 +709,7 @@ _message_iter_append_multi(DBusMessageIter *appender,
                                                 contents, &more);
         }
 
-        Py_DECREF(contents);
+        Py_CLEAR(contents);
         if (ret < 0) {
             break;
         }
@@ -695,13 +726,13 @@ _message_iter_append_multi(DBusMessageIter *appender,
 
     /* This must be run as cleanup, even on failure. */
     DBG("Closing %c container", container);
-    if (!dbus_message_iter_close_container(appender, &sub_appender)) {
+    if (!dbuspy_message_iter_close_container(appender, &sub_appender, (ret == 0))) {
         PyErr_NoMemory();
         ret = -1;
     }
 
 out:
-    Py_XDECREF(iterator);
+    Py_CLEAR(iterator);
     dbus_free(sig);
     return ret;
 }
@@ -819,7 +850,7 @@ _message_iter_append_variant(DBusMessageIter *appender, PyObject *obj)
     }
 
 out:
-    Py_XDECREF(obj_sig);
+    Py_CLEAR(obj_sig);
     return ret;
 }
 
@@ -1021,10 +1052,7 @@ dbus_py_Message_append(Message *self, PyObject *args, PyObject *kwargs)
     PyObject *signature_obj = NULL;
     DBusSignatureIter sig_iter;
     DBusMessageIter appender;
-    int i;
     static char *argnames[] = {"signature", NULL};
-    /* must start FALSE for the case where there's nothing there and we
-     * never iterate at all */
     dbus_bool_t more;
 
     if (!self->msg) return DBusPy_RaiseUnusableMessage();
@@ -1057,24 +1085,35 @@ dbus_py_Message_append(Message *self, PyObject *args, PyObject *kwargs)
         PyErr_SetString(PyExc_ValueError, "Corrupt type signature");
         goto err;
     }
-    dbus_signature_iter_init(&sig_iter, signature);
     dbus_message_iter_init_append(self->msg, &appender);
-    more = (signature[0] != '\0');
-    for (i = 0; i < PyTuple_GET_SIZE(args); i++) {
-        if (_message_iter_append_pyobject(&appender, &sig_iter,
-                                          PyTuple_GET_ITEM(args, i),
-                                          &more) < 0) {
+
+    if (signature[0] != '\0') {
+        int i = 0;
+
+        more = TRUE;
+        dbus_signature_iter_init(&sig_iter, signature);
+        while (more) {
+            if (i >= PyTuple_GET_SIZE(args)) {
+                PyErr_SetString(PyExc_TypeError, "More items found in D-Bus "
+                                "signature than in Python arguments");
+                goto hosed;
+            }
+            if (_message_iter_append_pyobject(&appender, &sig_iter,
+                                              PyTuple_GET_ITEM(args, i),
+                                              &more) < 0) {
+                goto hosed;
+            }
+            i++;
+        }
+        if (i < PyTuple_GET_SIZE(args)) {
+            PyErr_SetString(PyExc_TypeError, "Fewer items found in D-Bus "
+                    "signature than in Python arguments");
             goto hosed;
         }
     }
-    if (more) {
-        PyErr_SetString(PyExc_TypeError, "More items found in D-Bus "
-                        "signature than in Python arguments");
-        goto hosed;
-    }
 
     /* success! */
-    Py_XDECREF(signature_obj);
+    Py_CLEAR(signature_obj);
     Py_RETURN_NONE;
 
 hosed:
@@ -1085,7 +1124,7 @@ hosed:
     dbus_message_unref(self->msg);
     self->msg = NULL;
 err:
-    Py_XDECREF(signature_obj);
+    Py_CLEAR(signature_obj);
     return NULL;
 }
 
